Type: Package
Package: Rage
Title: Life History Metrics from Matrix Population Models
Version: 1.9.0
Authors@R: c(
    person("Patrick", "Barks", , "barks@biology.sdu.dk", role = "aut",
           comment = c(ORCID = "0000-0002-5947-8151")),
    person("Danny", "Buss", , "dlb50@cam.ac.uk", role = "ctb"),
    person("Pol", "Capdevila", , "pcapdevila.pc@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-2842-4302")),
    person("Hal", "Caswell", , "h.caswell@uva.nl", role = "aut",
           comment = c(ORCID = "0000-0003-4394-6894")),
    person(c("Judy", "P."), "Che-Castaldo", , "jchecastaldo@lpzoo.org", role = "aut",
           comment = c(ORCID = "0000-0002-9118-9202")),
    person(c("Richard", "A."), "Hinrichsen", , "rich@hinrichsenenvironmental.com", role = "aut",
           comment = c(ORCID = "0000-0003-0761-3005")),
    person("John", "Jackson", , "jjackson0308@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-4563-2840")),
    person("Tamora", "James", , "tamoradjames@protonmail.com", role = "aut",
           comment = c(ORCID = "0000-0003-1363-4742")),
    person("Owen", "Jones", , "jones@biology.sdu.dk", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-5720-4686")),
    person("Sam", "Levin", , "levisc8@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-3289-9925")),
    person("William K.", "Petry", , "wpetry@ncsu.edu", role = "aut",
           comment = c(ORCID = "0000-0002-5230-5987")),
    person("Roberto", "Salguero-Gomez", , "rob.salguero@zoo.ox.ac.uk", role = "aut",
           comment = c(ORCID = "0000-0002-6085-4433")),
    person("Caroline", "Schuette", , "cschuette17@gmail.com", role = "ctb",
           comment = c(ORCID = "0000-0002-2063-8736")),
    person("Iain", "Stott", , "stott@biology.sdu.dk", role = "aut",
           comment = c(ORCID = "0000-0003-2724-7436")),
    person(c("Chelsea", "C."), "Thomas", , "esochels@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-8155-9353")),
    person(c("Christina", "M."), "Hernández", , "cmh352@cornell.edu", role = "aut",
           comment = c(ORCID = "0000-0002-7188-8217")),
    person("Lotte", "de Vries", , "c.devries@uva.nl", role = "aut",
           comment = c(ORCID = "0000-0001-8955-0479")),
    person("Stefano", "Giaimo", , "giaimo@evolbio.mpg.de", role = "aut",
           comment = c(ORCID = "0000-0003-0421-3065"))
  )
Description: Functions for calculating life history metrics using matrix
    population models ('MPMs'). Described in Jones et al. (2021)
    <doi:10.1101/2021.04.26.441330>.
License: GPL-3
URL: https://github.com/jonesor/Rage, https://jonesor.github.io/Rage/
BugReports: https://github.com/jonesor/Rage/issues
Depends: R (>= 3.5.0)
Imports: DiagrammeR, expm, MASS, popdemo, stats, utils
Suggests: DiagrammeRsvg, ggplot2, knitr, Rcompadre, rmarkdown, rsvg,
        spelling, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/testthat/parallel: false
Encoding: UTF-8
Language: en-GB
LazyData: true
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-04-23 15:01:34 UTC; jones
Author: Patrick Barks [aut] (ORCID: <https://orcid.org/0000-0002-5947-8151>),
  Danny Buss [ctb],
  Pol Capdevila [aut] (ORCID: <https://orcid.org/0000-0002-2842-4302>),
  Hal Caswell [aut] (ORCID: <https://orcid.org/0000-0003-4394-6894>),
  Judy P. Che-Castaldo [aut] (ORCID:
    <https://orcid.org/0000-0002-9118-9202>),
  Richard A. Hinrichsen [aut] (ORCID:
    <https://orcid.org/0000-0003-0761-3005>),
  John Jackson [aut] (ORCID: <https://orcid.org/0000-0002-4563-2840>),
  Tamora James [aut] (ORCID: <https://orcid.org/0000-0003-1363-4742>),
  Owen Jones [aut, cre] (ORCID: <https://orcid.org/0000-0001-5720-4686>),
  Sam Levin [aut] (ORCID: <https://orcid.org/0000-0002-3289-9925>),
  William K. Petry [aut] (ORCID: <https://orcid.org/0000-0002-5230-5987>),
  Roberto Salguero-Gomez [aut] (ORCID:
    <https://orcid.org/0000-0002-6085-4433>),
  Caroline Schuette [ctb] (ORCID:
    <https://orcid.org/0000-0002-2063-8736>),
  Iain Stott [aut] (ORCID: <https://orcid.org/0000-0003-2724-7436>),
  Chelsea C. Thomas [aut] (ORCID:
    <https://orcid.org/0000-0002-8155-9353>),
  Christina M. Hernández [aut] (ORCID:
    <https://orcid.org/0000-0002-7188-8217>),
  Lotte de Vries [aut] (ORCID: <https://orcid.org/0000-0001-8955-0479>),
  Stefano Giaimo [aut] (ORCID: <https://orcid.org/0000-0003-0421-3065>)
Maintainer: Owen Jones <jones@biology.sdu.dk>
Repository: CRAN
Date/Publication: 2026-04-23 15:30:02 UTC
Built: R 4.6.0; ; 2026-04-29 22:21:46 UTC; unix
