# Generated by roxygen2: do not edit by hand

export(entropy_d)
export(entropy_k)
export(entropy_k_age)
export(entropy_k_stage)
export(evenness_shannon)
export(gen_time)
export(hx_to_lx)
export(hx_to_px)
export(is_leslie_matrix)
export(leslie_collapse)
export(life_elas)
export(life_expect_mean)
export(life_expect_var)
export(longevity)
export(lx_to_hx)
export(lx_to_px)
export(mature_age)
export(mature_distrib)
export(mature_prob)
export(mpm_collapse)
export(mpm_rearrange)
export(mpm_split)
export(mpm_standardise)
export(mpm_standardize)
export(mpm_to_hx)
export(mpm_to_lx)
export(mpm_to_mx)
export(mpm_to_px)
export(mpm_to_table)
export(name_stages)
export(net_repro_rate)
export(perturb_matrix)
export(perturb_stochastic)
export(perturb_trans)
export(perturb_vr)
export(plot_life_cycle)
export(pop_vectors)
export(px_to_hx)
export(px_to_lx)
export(qsd_converge)
export(repro_stages)
export(scale_mpm_to_lambda1)
export(shape_rep)
export(shape_surv)
export(stage_at_death_dist)
export(standard_stages)
export(vital_rates)
export(vr_dorm_enter)
export(vr_dorm_exit)
export(vr_fecundity)
export(vr_growth)
export(vr_mat_R)
export(vr_mat_U)
export(vr_shrinkage)
export(vr_stasis)
export(vr_survival)
export(vr_vec_dorm_enter)
export(vr_vec_dorm_exit)
export(vr_vec_growth)
export(vr_vec_reproduction)
export(vr_vec_shrinkage)
export(vr_vec_stasis)
export(vr_vec_survival)
importFrom(DiagrammeR,grViz)
importFrom(MASS,ginv)
importFrom(expm,"%^%")
importFrom(popdemo,isErgodic)
importFrom(popdemo,project)
importFrom(stats,na.omit)
importFrom(stats,runif)
importFrom(utils,head)
