#' Example matrix population model (MPM)
#'
#' @description
#' An example matrix population model (MPM) used for demonstration and testing
#' purposes. The MPM consists of five stage classes: 'seed', 'small', 'medium',
#' 'large', and 'dormant'.
#'
#' @format
#' A list with two elements:
#' \describe{
#'   \item{matU}{The survival-related component of the MPM.}
#'   \item{matF}{The sexual reproduction component of the MPM.}
#' }
"mpm1"

#' Example Leslie matrix population model (MPM)
#'
#' @description
#' An example Leslie matrix population model (MPM) used for demonstration and
#' testing purposes.
#'
#' @format
#' A list with two elements:
#' \describe{
#'   \item{matU}{The survival-related component of the MPM.}
#'   \item{matF}{The sexual reproduction component of the MPM.}
#' }
"leslie_mpm1"
