complete -c rage -l max-work-factor -d 'Maximum work factor to allow for passphrase decryption.' -r
complete -c rage -s r -l recipient -d 'Encrypt to the specified RECIPIENT. May be repeated.' -r
complete -c rage -s R -l recipients-file -d 'Encrypt to the recipients listed at PATH. May be repeated.' -r -F
complete -c rage -s i -l identity -d 'Use the identity file at IDENTITY. May be repeated.' -r -F
complete -c rage -s j -d 'Use age-plugin-PLUGIN-NAME in its default mode as an identity.' -r
complete -c rage -s o -l output -d 'Write the result to the file at path OUTPUT.' -r -F
complete -c rage -s h -l help -d 'Print this help message and exit.'
complete -c rage -s V -l version -d 'Print version info and exit.'
complete -c rage -s e -l encrypt -d 'Encrypt the input (the default).'
complete -c rage -s d -l decrypt -d 'Decrypt the input.'
complete -c rage -s p -l passphrase -d 'Encrypt with a passphrase instead of recipients.'
complete -c rage -s a -l armor -d 'Encrypt to a PEM encoded format.'
