complete -c rage-keygen -s o -l output -d 'Write the result to the file at path OUTPUT. Defaults to standard output.' -r -f -a "(__fish_complete_directories)"
complete -c rage-keygen -s h -l help -d 'Print this help message and exit.'
complete -c rage-keygen -s V -l version -d 'Print version info and exit.'
complete -c rage-keygen -s y -d 'Convert an identity file to a recipients file.'
